var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
import { Component, Inject, QueryList, ViewChildren } from '@angular/core';
import { FormControl } from '@angular/forms';
import { EventManager } from '@angular/platform-browser';
import { StateService } from './shared/state.service';
import { UrlService } from './shared/url.service';
import { fuzzySearch } from './shared/fuzzy-search.function';
import { LevenshteinDistance } from './shared/levenshtein-distance';
import { SandboxLoader } from './shared/sandbox-loader';
import { MIDDLEWARE } from '../lib/middlewares';
import { Observable } from 'rxjs';
import { debounceTime, distinctUntilChanged } from 'rxjs/operators';
let AppComponent = class AppComponent {
    constructor(stateService, urlService, eventManager, levenshteinDistance, middleware) {
        this.stateService = stateService;
        this.urlService = urlService;
        this.eventManager = eventManager;
        this.levenshteinDistance = levenshteinDistance;
        this.middleware = middleware;
        this.commandBarActive = false;
        this.commandBarPreview = false;
        this.selectedSandboxAndScenarioKeys = { sandboxKey: null, scenarioKey: null };
        this.filter = new FormControl();
        this.shortcuts = this.getShortcuts();
    }
    ngOnInit() {
        const sandboxMenuItems = SandboxLoader.getSandboxMenuItems();
        this.middleware
            .subscribe(middleware => this.activeMiddleware = middleware);
        if (this.urlService.embed) {
            this.selectedSandboxAndScenarioKeys = {
                sandboxKey: this.urlService.select.sandboxKey,
                scenarioKey: this.urlService.select.scenarioKey,
            };
        }
        else {
            if (this.urlService.select) {
                this.selectedSandboxAndScenarioKeys = {
                    sandboxKey: this.urlService.select.sandboxKey,
                    scenarioKey: this.urlService.select.scenarioKey,
                };
            }
            this.eventManager.addGlobalEventListener('window', 'keydown.control.p', this.blockEvent);
            this.eventManager.addGlobalEventListener('window', 'keydown.F2', this.blockEvent);
            this.eventManager.addGlobalEventListener('window', 'keyup.control.p', (event) => {
                this.blockEvent(event);
                this.toggleCommandBar();
            });
            this.eventManager.addGlobalEventListener('window', 'keyup.F2', (event) => {
                this.blockEvent(event);
                this.toggleCommandBar();
            });
            let filterValue = this.stateService.getFilter();
            this.totalSandboxes = sandboxMenuItems.length;
            this.filteredSandboxMenuItems = this.filterSandboxes(sandboxMenuItems, filterValue);
            this.filter.setValue(filterValue);
            this.filter.valueChanges.pipe(debounceTime(300), distinctUntilChanged())
                .subscribe(value => {
                this.stateService.setFilter(value);
                this.filteredSandboxMenuItems = this.filterSandboxes(sandboxMenuItems, value);
                if (!value) {
                    this.selectScenario(null, null);
                }
            });
        }
    }
    onFilterBoxArrowDown(event, switchToScenario = false) {
        event.preventDefault();
        let elementRef;
        const currentIndex = this.findCurrentScenarioIndex();
        if (currentIndex) {
            elementRef = this.focusScenarioLinkElement(currentIndex + 1);
        }
        else {
            elementRef = this.focusScenarioLinkElement(0);
        }
        if (switchToScenario) {
            this.showScenario(elementRef);
        }
    }
    onFilterBoxArrowUp(event, switchToScenario = false) {
        event.preventDefault();
        let elementRef;
        const currentIndex = this.findCurrentScenarioIndex();
        if (currentIndex) {
            elementRef = this.focusScenarioLinkElement(currentIndex - 1);
        }
        else if (this.scenarioLinkElements.length > 0) {
            elementRef = this.focusScenarioLinkElement(this.scenarioLinkElements.length - 1);
        }
        if (switchToScenario) {
            this.showScenario(elementRef);
        }
    }
    onScenarioLinkKeyDown(scenarioElement, filterElement, event) {
        event.preventDefault();
        switch (event.key) {
            case 'Up':
            case 'ArrowUp':
                this.goUp(scenarioElement);
                break;
            case 'Down':
            case 'ArrowDown':
                this.goDown(scenarioElement);
                break;
            default:
                if (event.key !== 'Escape' && event.key !== 'Enter') {
                    if (event.key.length === 1) {
                        this.filter.setValue(`${this.filter.value}${event.key}`);
                    }
                    filterElement.focus();
                }
                break;
        }
    }
    onScenarioLinkKeyUp(scenarioElement, event) {
        event.preventDefault();
        switch (event.key) {
            case 'Escape':
                this.commandBarActive = false;
                break;
            case 'Enter':
                scenarioElement.click();
                break;
        }
    }
    onScenarioLinkControlDown(scenarioElement, event) {
        if (!this.commandBarActive)
            return;
        event.preventDefault();
        let elementRef = this.goDown(scenarioElement);
        this.showScenario(elementRef);
    }
    onScenarioLinkControlUp(scenarioElement, event) {
        if (!this.commandBarActive)
            return;
        event.preventDefault();
        let elementRef = this.goUp(scenarioElement);
        this.showScenario(elementRef);
    }
    onCommandBarStartPreview(event) {
        event.preventDefault();
        this.commandBarPreview = true;
    }
    onCommandBarStopPreview() {
        this.commandBarPreview = false;
    }
    onScenarioClick(sandboxKey, scenarioKey, event) {
        event.preventDefault();
        this.selectScenario(sandboxKey, scenarioKey);
    }
    isSelected(sandbox, scenario) {
        return this.selectedSandboxAndScenarioKeys.scenarioKey === scenario.key
            && this.selectedSandboxAndScenarioKeys.sandboxKey.toLowerCase() === sandbox.key.toLowerCase();
    }
    toggleCommandBar() {
        this.commandBarActive = !this.commandBarActive;
    }
    blockEvent(e) {
        e.preventDefault();
        e.stopPropagation();
    }
    showScenario(selectedScenarioElementRef) {
        if (selectedScenarioElementRef) {
            this.selectScenario(selectedScenarioElementRef.nativeElement.getAttribute('sandboxMenuItemKey'), parseInt(selectedScenarioElementRef.nativeElement.getAttribute('scenarioMenuItemkey'), 10));
        }
    }
    findCurrentScenarioIndex() {
        let currentIndex;
        if (this.scenarioLinkElements.length > 0) {
            this.scenarioLinkElements.map((element, i) => {
                if (element.nativeElement.className.includes('selected')) {
                    currentIndex = i;
                }
            });
        }
        return currentIndex;
    }
    goUp(scenarioElement) {
        let currentIndex = -1;
        this.scenarioLinkElements.forEach((scenarioElementRef, index) => {
            if (scenarioElementRef.nativeElement === scenarioElement) {
                currentIndex = index;
            }
        });
        if (currentIndex === 0) {
            return this.focusScenarioLinkElement(this.scenarioLinkElements.length - 1);
        }
        else {
            return this.focusScenarioLinkElement(currentIndex - 1);
        }
    }
    goDown(scenarioElement) {
        let currentIndex = -1;
        this.scenarioLinkElements.forEach((scenarioElementRef, index) => {
            if (scenarioElementRef.nativeElement === scenarioElement) {
                currentIndex = index;
            }
        });
        if (currentIndex === this.scenarioLinkElements.length - 1) {
            return this.focusScenarioLinkElement(0);
        }
        else {
            return this.focusScenarioLinkElement(currentIndex + 1);
        }
    }
    focusScenarioLinkElement(index) {
        if (this.scenarioLinkElements.toArray()[index]) {
            let elementRef = this.scenarioLinkElements.toArray()[index];
            elementRef.nativeElement.focus();
            return elementRef;
        }
    }
    filterSandboxes(sandboxMenuItems, filter) {
        if (!filter) {
            return sandboxMenuItems.map((item, i) => Object.assign({}, item, { tabIndex: i }));
        }
        let tabIndex = 0;
        let filterNormalized = filter.toLowerCase();
        return sandboxMenuItems
            .reduce((accum, curr) => {
            let searchKeyNormalized = curr.searchKey.toLowerCase();
            let indexMatches = fuzzySearch(filterNormalized, searchKeyNormalized);
            if (indexMatches) {
                let weight = this.levenshteinDistance.getDistance(filterNormalized, searchKeyNormalized);
                return [...accum, Object.assign({}, curr, { weight, indexMatches })];
            }
            else {
                return accum;
            }
        }, [])
            .sort((a, b) => {
            return a.weight - b.weight;
        })
            .map(sandboxMenuItem => Object.assign({}, sandboxMenuItem, { tabIndex: tabIndex++ }));
    }
    selectScenario(sandboxKey, scenarioKey) {
        this.selectedSandboxAndScenarioKeys = { sandboxKey, scenarioKey };
        this.urlService.setSelected(sandboxKey, scenarioKey);
    }
    getShortcuts() {
        return [
            {
                keys: ['ctrl + p', 'f2'],
                description: 'Toggle command bar open/closed',
            },
            {
                keys: ['esc'],
                description: 'Close command bar',
            },
            {
                keys: ['\u2191', '\u2193'],
                description: 'Navigate up or down in command bar list',
            },
            {
                keys: ['ctrl + \u2191', 'ctrl + \u2193'],
                description: 'Switch scenarios while navigating up or down in command bar list',
            },
        ];
    }
};
__decorate([
    ViewChildren('scenarioElement'),
    __metadata("design:type", QueryList)
], AppComponent.prototype, "scenarioLinkElements", void 0);
AppComponent = __decorate([
    Component({
        selector: 'ap-root',
        template: `
      <div class="shield" *ngIf="commandBarActive" (click)="toggleCommandBar()"></div>
      <div class="command-bar" *ngIf="filteredSandboxMenuItems"
           (keydown.control)="onCommandBarStartPreview($event)"
           (keyup.control)="onCommandBarStopPreview()"
           [class.command-bar--open]="commandBarActive"
           [class.command-bar--preview]="commandBarPreview">
          <input
              class="command-bar__filter"
              type="text"
              name="filter"
              placeholder="search..."
              [formControl]="filter"
              #filterElement
              [apFocus]="commandBarActive"
              (keyup.Esc)="commandBarActive = false"
              (keydown.arrowup)="onFilterBoxArrowUp($event)"
              (keydown.arrowdown)="onFilterBoxArrowDown($event)"
              (keydown.control.arrowup)="onFilterBoxArrowUp($event, true)"
              (keydown.control.arrowdown)="onFilterBoxArrowDown($event, true)">
          <div *ngIf="filteredSandboxMenuItems && filteredSandboxMenuItems.length > 0" class="command-bar__sandboxes">
              <div class="command-bar__sandbox"
                   *ngFor="let sandboxMenuItem of filteredSandboxMenuItems; index as menuItemIndex">
                  <h2 class="command-bar__title" title="{{sandboxMenuItem.label}} {{sandboxMenuItem.name}}"
                      [class.command-bar__sandbox-title--selected]="selectedSandboxAndScenarioKeys.sandboxKey === sandboxMenuItem.key">
                      <span class="command-bar__name"
                            [innerHtml]="sandboxMenuItem.name | apHighlightSearchMatch : sandboxMenuItem.indexMatches"></span>
                      <span class="command-bar__label" *ngIf="sandboxMenuItem.label"
                            [innerHtml]="sandboxMenuItem.label | apHighlightSearchMatch : sandboxMenuItem.indexMatches : sandboxMenuItem.name.length"></span>
                  </h2>
                  <div class="command-bar__scenarios"
                       *ngFor="let scenarioMenuItem of sandboxMenuItem.scenarioMenuItems; index as scenarioMenuItemIndex">
                      <a
                          class="command-bar__scenario-link"
                          #scenarioElement
                          [tabindex]="scenarioMenuItem.tabIndex"
                          [attr.sandboxMenuItemKey]="sandboxMenuItem.key"
                          [attr.scenarioMenuItemkey]="scenarioMenuItem.key"
                          (keydown.control.arrowup)="onScenarioLinkControlUp(scenarioElement, $event)"
                          (keydown.control.arrowdown)="onScenarioLinkControlDown(scenarioElement, $event)"
                          (keydown.arrowup)="onScenarioLinkKeyDown(scenarioElement, filterElement, $event)"
                          (keydown.arrowdown)="onScenarioLinkKeyDown(scenarioElement, filterElement, $event)"
                          (keydown.esc)="onScenarioLinkKeyDown(scenarioElement, filterElement, $event)"
                          (keydown.enter)="onScenarioLinkKeyDown(scenarioElement, filterElement, $event)"
                          (keyup.esc)="onScenarioLinkKeyUp(scenarioElement, $event)"
                          (keyup.enter)="onScenarioLinkKeyUp(scenarioElement, $event)"
                          (click)="onScenarioClick(sandboxMenuItem.key, scenarioMenuItem.key, $event); toggleCommandBar()"
                          [class.command-bar__scenario-link--selected]="isSelected(sandboxMenuItem, scenarioMenuItem)">
                          <ap-pin [selected]="isSelected(sandboxMenuItem, scenarioMenuItem)"></ap-pin>
                          <span class="command-bar__scenario-label">
                        {{scenarioMenuItem.description}}
                      </span>
                      </a>
                  </div>
              </div>
          </div>
          <a *ngIf="filteredSandboxMenuItems && filteredSandboxMenuItems.length > 0" class="command-bar__brand"
             href="http://www.angularplayground.it/" target="_blank">
              <ap-logo></ap-logo>
          </a>
      </div>
      <section class="content">
          <ap-drawer *ngIf="activeMiddleware.overlay" (openCommandBarClick)="toggleCommandBar()" [class.content__menu--hide]="commandBarActive" class="content__menu"></ap-drawer>
          <div class="content__none" *ngIf="!selectedSandboxAndScenarioKeys.sandboxKey">
              <div class="content__none-message">
                  <p *ngIf="totalSandboxes > 0">
                      The playground has {{totalSandboxes}} sandboxed component{{totalSandboxes > 1 ? 's' : ''}}
                  </p>
                  <p *ngIf="totalSandboxes === 0">
                      Get started - create your first sandbox! <a href="http://www.angularplayground.it/docs/how-to/sandboxing-components" target="_blank">Click here</a>.
                  </p>
                  <div class="content__shortcuts">
                      <div class="content__shortcut" *ngFor="let shortcut of shortcuts">
                          <div class="content__shortcut-label">
                              <ng-container *ngFor="let key of shortcut.keys; let i = index">
                                  <code>
                                      {{key}}
                                  </code>
                                  <ng-container *ngIf="shortcut.keys.length > 1 && i < shortcut.keys.length - 1">&nbsp;&nbsp;/&nbsp;&nbsp;</ng-container>
                              </ng-container>
                          </div>
                          <div class="content__shortcut-value">
                              {{shortcut.description}}
                          </div>
                      </div>
                  </div>
              </div>
          </div>
          <ng-container *ngIf="selectedSandboxAndScenarioKeys.sandboxKey">
              <ap-scenario [selectedSandboxAndScenarioKeys]="selectedSandboxAndScenarioKeys"></ap-scenario>
          </ng-container>
      </section>
    `,
        styles: [`
          /* Globals */
          * {
            box-sizing: border-box;
          }

          :host {
            display: flex;
            flex-direction: column;
          }

          /* Shield */
          .shield {
            height: 100vh;
            opacity: 0;
            position: absolute;
            z-index: 2;
            width: 100%;
          }

          /* Command Bar */
          .command-bar {
            background-color: #252526;
            box-shadow: 0 3px 8px 5px black;
            color: white;
            display: flex;
            flex-direction: column;
            font-family: Consolas, monospace;
            left: 50%;
            margin-top: -6px;
            max-height: 100vh;
            padding-top: 10px;
            position: absolute;
            transform: translate(-50%, -120%);
            transition: transform ease 100ms, opacity ease 100ms;
            width: 376px;
            z-index: 9999999999999;
          }

          .command-bar::before {
            border-bottom: solid 1px black;
            content: "";
            display: block;
            position: absolute;
            top: 61px;
            width: 100%;
          }

          .command-bar--open {
            min-height: 60px;
            transform: translate(-50%, 0);
          }

          .command-bar--preview {
            opacity: .7;
          }

          .command-bar__filter {
            background-color: #3c3c3c;
            border: 1px solid #174a6c;
            color: white;
            font-family: Consolas, monospace;
            font-size: 16px;
            margin: 6px 0 0 5px;
            padding: 8px;
            width: 365px;
            z-index: 1;
          }

          .command-bar__filter::placeholder {
            color: #a9a9a9;
          }

          .command-bar__filter:-ms-input-placeholder {
            color: #a9a9a9;
          }

          .command-bar__filter::-moz-focus-inner {
            border: 0;
            padding: 0;
          }

          /* Sandboxes */
          .command-bar__sandboxes {
            border-top: solid 1px rgba(255, 255, 255, .1);
            margin-top: 9px;
            overflow: auto;
            position: relative;
            max-height: calc(100vh - 109px);
          }

          .command-bar__sandboxes::-webkit-scrollbar {
            background-color: transparent;
            width: 6px;
          }

          .command-bar__sandboxes::-webkit-scrollbar-track {
            border-left: solid 1px black;
            background: rgba(255, 255, 255, 0.1);
          }

          .command-bar__sandboxes::-webkit-scrollbar-thumb  {
            background-color: rgba(255, 255, 255, 0.1);
            margin-left: 2px;
            width: 4px;
          }

          .command-bar__sandbox {
            border-bottom: solid 1px black;
            border-top: solid 1px rgba(255, 255, 255, .1);
            padding: 8px 6px 14px;
          }

          .command-bar__sandbox:first-child {
            border-top: none;
          }

          .command-bar__sandbox:last-child {
            border-bottom: none;
            padding-bottom: 12px;
          }

          .command-bar__title {
            align-items: center;
            color: rgba(255, 255, 255, .5);
            display: flex;
            font-family: Consolas, monospace;
            font-size: 12px;
            font-weight: normal;
            justify-content: space-between;
            margin: 0 0 5px;
            padding: 5px 0 0;
          }

          .command-bar__title ::ng-deep mark {
            background: transparent;
            color: #0097fb;
            font-weight: bold;
          }

          .command-bar__name {
            max-width: 100%;
            overflow: hidden;
            text-overflow: ellipsis;
            white-space: nowrap;
          }

          .command-bar__label {
            background: rgba(255, 255, 255, .1);
            border-radius: 2px;
            display: block;
            font-size: 10px;
            margin-left: 10px;
            padding: 4px 5px 3px;
          }

          /* Scenarios */
          .command-bar__scenarios {
            display: flex;
          }

          .command-bar__scenario-link {
            align-items: center;
            border-radius: 2px;
            color: rgba(255, 255, 255, .5);
            cursor: pointer;
            display: flex;
            padding: 4px 3px;
            width: 100%;
          }

          .command-bar__scenario-link:hover,
          .command-bar__scenario-link:active,
          .command-bar__scenario-link:focus {
            background-color: #0097fb;
            color: white;
            outline-style: none;
          }

          .command-bar__scenario-link:hover .command-bar__scenario-icon,
          .command-bar__scenario-link:active .command-bar__scenario-icon,
          .command-bar__scenario-link:focus .command-bar__scenario-icon {
            opacity: .5;
          }

          .command-bar__scenario-label {
            line-height: 1;
            max-width: calc(100% - 26px);
            min-width: calc(100% - 26px);
            padding-bottom: 2px;
          }

          .command-bar__scenario-link--selected {
            background: rgba(255, 255, 255, .1);
            color: white;
          }

          .command-bar__scenario-link:hover .command-bar__scenario-icon--selected,
          .command-bar__scenario-link:active .command-bar__scenario-icon--selected,
          .command-bar__scenario-link:focus .command-bar__scenario-icon--selected {
            fill: white;
          }

          /* Brand */
          .command-bar__brand {
            border-top: solid 1px black;
            display: block;
            position: relative;
            padding: 9px 0 3px;
            text-align: center;
          }

          .command-bar__brand::before {
            border-bottom: solid 1px rgba(255, 255, 255, .1);
            content: "";
            display: block;
            left: 0;
            position: absolute;
            top: 0;
            width: 100%;
          }

          .command-bar__brand:hover .command-bar__logo__box {
            fill: rgba(255, 255, 255, .2);
          }

          .command-bar__brand:hover .command-bar__logo__letter {
            fill: rgba(255, 255, 255, .75);
          }

          /* Content */
          .content__none {
            align-items: center;
            border: 0;
            display: flex;
            min-height: calc(100vh - 4em);
            justify-content: center;
            padding-top: 2em;
            padding-bottom: 2em;
            position: relative;
            width: 100%;
          }

          .content__none-message {
            font-family: Arial, sans-serif;
            max-width: 50%;
            min-width: 450px;
            text-align: center;
          }

          .content__none-message em {
            color: #666;
          }

          .content__none-message p {
            font-size: 20px;
          }

          .content__shortcuts {
            border-top: solid 1px #ccc;
            margin-top: 2em;
            padding: 30px 0 0 100px;
            width: 520px;
          }

          .content__shortcut {
            display: flex;
          }

          .content__shortcut-label {
            align-items: center;
            display: flex;
            font-size: 11px;
            justify-content: flex-end;
            max-width: 150px;
            min-width: 150px;
            padding: 8px 12px 8px 0;
            white-space: nowrap;
          }

          .content__shortcut-label code {
            background: #eee;
            border: solid 1px #ccc;
            border-radius: 4px;
            padding: 3px 7px;
          }

          .content__shortcut-value {
            align-items: center;
            display: flex;
            font-size: 11px;
            line-height: 1.75;
            text-align: left;
            white-space: nowrap;
          }
    `],
    }),
    __param(4, Inject(MIDDLEWARE)),
    __metadata("design:paramtypes", [StateService,
        UrlService,
        EventManager,
        LevenshteinDistance,
        Observable])
], AppComponent);
export { AppComponent };
//# sourceMappingURL=app.component.js.map